﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2013 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion


using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.DataAnalysis.Transformations {
  [Item("LinearTransformation", "Represents a linear transformation with multiplication and addition.")]
  public class LinearTransformation : Transformation<double> {
    protected const string MultiplicandParameterName = "Multiplicand";
    protected const string SummandParameterName = "Summand";

    #region Parameters
    public IValueParameter<DoubleValue> MultiplicandParameter {
      get { return (IValueParameter<DoubleValue>)Parameters[MultiplicandParameterName]; }
    }
    public IValueParameter<DoubleValue> SummandParameter {
      get { return (IValueParameter<DoubleValue>)Parameters[SummandParameterName]; }
    }
    #endregion

    #region properties
    public double Multiplicand {
      get { return MultiplicandParameter.Value.Value; }
    }

    public double Summand {
      get { return SummandParameter.Value.Value; }
    }
    #endregion

    [StorableConstructor]
    protected LinearTransformation(bool deserializing) : base(deserializing) { }
    protected LinearTransformation(Transformation<double> original, Cloner cloner)
      : base(original, cloner) {
    }
    protected LinearTransformation() {
    }
    public LinearTransformation(IEnumerable<string> allowedColumns)
      : base(allowedColumns) {
      Parameters.Add(new ValueParameter<DoubleValue>(MultiplicandParameterName, "Multiplicand for linear transformation", new DoubleValue(1.0)));
      Parameters.Add(new ValueParameter<DoubleValue>(SummandParameterName, "Summand for linear transformation", new DoubleValue(0.0)));
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new LinearTransformation(this, cloner);
    }

    public override IEnumerable<double> Apply(IEnumerable<double> data) {
      return data.Select(e => e * Multiplicand + Summand);
    }

    public override bool Check(IEnumerable<double> data, out string errorMsg) {
      StringBuilder sb = new StringBuilder();
      bool success = true;
      errorMsg = null;

      if (Multiplicand == 0.0) {
        sb.Append(String.Format("Multiplicand is 0, all {0} entries will be set to {1}. Reversal will not be possble", data.Count(), Summand));
        errorMsg = sb.ToString();
        success = false;
      }
      return success;
    }
  }
}
